//
//  ZegoLiveRoomDefines.h
//  zegoliveroom
//
//  Copyright © 2017年 ZEGO. All rights reserved.
//

#ifndef ZegoLiveRoomDefines_h
#define ZegoLiveRoomDefines_h

#include "./AVDefines.h"
#include "./RoomDefines.h"


namespace ZEGO
{
    namespace LIVEROOM
    {
        using namespace COMMON;
        using COMMON::ZEGONetType;
        
        using COMMON::ZegoRoomRole;
        
        using COMMON::ZegoStreamInfo;
        
        using COMMON::ZegoStreamUpdateType;
        
        const int kInvalidSeq = -1;
        
        /** 本地预览视频视图的模式 */
        enum ZegoVideoViewMode
        {
            ZegoVideoViewModeScaleAspectFit = 0,    /**< 等比缩放，可能有黑边，SDK 默认值 */
            ZegoVideoViewModeScaleAspectFill = 1,   /**< 等比缩放填充整View，可能有部分被裁减 */
            ZegoVideoViewModeScaleToFill = 2,       /**< 填充整个View */
        };
        
        struct ZegoPublishQuality
        {
            double cfps;            ///< 视频帧率(采集)
            double vencFps;         ///< 视频帧率(编码)
            double fps;             ///< 视频帧率(网络发送)
            double kbps;            ///< 视频码率(kb/s)
            
            double acapFps;         ///< 音频帧率（采集）
            double afps;            ///< 音频帧率（网络发送）
            double akbps;           ///< 音频码率(kb/s)
        
            int rtt;                ///< 延时(ms)
            int pktLostRate;        ///< 丢包率(0~255)
            int quality;            ///< 质量(0~3)
            
            bool isHardwareVenc;    ///< 是否硬编
            int videoCodecId;       ///< 视频编码格式(参考ZegoVideoCodecAvc)
            int width;              ///< 视频宽度
            int height;             ///< 视频高度

            double totalBytes; ///< 已发送的总字节数，包括音频、视频及媒体次要信息等
            double audioBytes; ///< 已发送的音频字节数
            double videoBytes; ///< 已发送的视频字节数
            
            double cpuAppUsage;    ///< 当前 APP 的 CPU 使用率
            double cpuTotalUsage;  ///< 当前系统的 CPU 使用率
            
            double memoryAppUsage;         ///< 当前 APP 的内存使用率
            double memoryTotalUsage;       ///< 当前系统的内存使用率
            double memoryAppUsed;          ///< 当前 APP 的内存使用量, 单位 MB(win返回的是app实际占用内存工作集=专用内存工作集+共享内存工作集)

            ZegoPublishQuality()
            {
                cfps = 0;   
                vencFps = 0;
                fps = 0;    
                kbps = 0;   
                    
                acapFps = 0;
                afps = 0;   
                akbps = 0;  
                rtt = 0;     
                pktLostRate = 0;  
                quality = 0;      
                        
                isHardwareVenc = false;
                videoCodecId = 0;
                width = 0;            
                height = 0;           
                totalBytes = 0;  
                audioBytes = 0;  
                videoBytes = 0;  
                    
                cpuAppUsage = 0;    
                cpuTotalUsage = 0;  
                    
                memoryAppUsage = 0;  
                memoryTotalUsage = 0;
                memoryAppUsed = 0;   
            }
        };
        
        struct ZegoPlayQuality
        {
            double fps;              ///< 视频帧率(网络接收)
            double vdjFps;           ///< 视频帧率(dejitter)
            double vdecFps;          ///< 视频帧率(解码)
            double vrndFps;          ///< 视频帧率(渲染)
            double kbps;             ///< 视频码率(kb/s)
            
            double afps;             ///< 音频帧率(网络接收)
            double adjFps;           ///< 音频帧率(dejitter)
            double adecFps;          ///< 音频帧率(解码)
            double arndFps;          ///< 音频帧率(渲染)
            double akbps;            ///< 音频码率(kb/s)
            
            double audioBreakRate;   ///< 音频卡顿次数
            double videoBreakRate;   ///< 视频卡顿次数
            int rtt;                 ///< 延时(ms)
            int pktLostRate;         ///< 丢包率(0~255)
            int peerToPeerDelay;     ///< 端到端延迟
            int peerToPeerPktLostRate; ///< 端到端丢包率(0~255)
            int quality;             ///< 质量(0~3)
            int delay;               ///< 语音延时(ms)
            double mos;              ///< 音质评分。-1 表示未知。小于 2 表示音质非常差，杂音频现，大量语义丢失，完全无法交流。2 到 2.5 表示音质很差，偶有杂音，部分语义丢失，难以交流。2.5 到 3 表示音质较差，卡顿频繁，需要集中精力才能听清。3 到 3.5 表示音质一般，偶有卡顿，不是非常流畅，需要一点注意力才能听清。3.5 到 4 表示音质较好，偶有音质损伤，单依然清晰。大于 4 表示音质佳，清晰流畅。
            
            bool isHardwareVdec;     ///< 是否硬解
            int videoCodecId;        ///< 视频解码格式(参考ZegoVideoCodecAvc)
            int width;               ///< 视频宽度
            int height;              ///< 视频高度

            double totalBytes; ///< 已接收的总字节数，包括音频、视频及媒体次要信息等
            double audioBytes; ///< 已接收的音频字节数
            double videoBytes; ///< 已接收的视频字节数
            
            double cpuAppUsage;        ///< 当前 APP 的 CPU 使用率
            double cpuTotalUsage;      ///< 当前系统的 CPU 使用率
            
            double memoryAppUsage;         ///< 当前 APP 的内存使用率
            double memoryTotalUsage;       ///< 当前系统的内存使用率
            double memoryAppUsed;          ///< 当前 APP 的内存使用量, 单位 MB

            int avTimestampDiff;       ///< 音画不同步, 单位毫秒, 小于0表示视频超前音频的毫秒数, 大于0表示视频滞后音频的毫秒数, 等于0表示无差别. 当绝对值小于200，可基本认为音画同步，当绝对值连续10秒大于200可以认为异常

            unsigned int audioCumulativeBreakCount; ///< 累计音频卡顿次数
            unsigned int audioCumulativeBreakTime;  ///< 累计音频卡顿时长(ms)
            unsigned int audioCumulativeDecodeTime; ///< 累计音频解码时长(ms)
            double       audioCumulativeBreakRate;  ///< 累计音频卡比例(%)
            unsigned int videoCumulativeBreakCount; ///< 累计视频卡顿次数
            unsigned int videoCumulativeBreakTime;  ///< 累计视频卡顿时长(ms)
            unsigned int videoCumulativeDecodeTime; ///< 累计视频解码时长(ms)
            double       videoCumulativeBreakRate;  ///< 累计视频卡比例(%)

            ZegoPlayQuality()
            {
                fps = 0;           
                vdjFps = 0;        
                vdecFps = 0;       
                vrndFps = 0;       
                kbps = 0;          
                    
                afps = 0;          
                adjFps = 0;        
                adecFps = 0;       
                arndFps = 0;       
                akbps = 0;         
                    
                audioBreakRate = 0;
                videoBreakRate = 0;
                rtt = 0;              
                pktLostRate = 0;      
                peerToPeerDelay = 0;  
                peerToPeerPktLostRate = 0;
                quality = 0;          
                delay = 0;        
                mos = -1;
                        
                isHardwareVdec = false;  
                videoCodecId = 0;
                width = 0;            
                height = 0;           
                totalBytes = 0;
                audioBytes = 0;
                videoBytes = 0;
                    
                cpuAppUsage = 0;
                cpuTotalUsage = 0;
                    
                memoryAppUsage = 0;
                memoryTotalUsage = 0;
                memoryAppUsed = 0;

                avTimestampDiff = 0;

                audioCumulativeBreakCount = 0;
                audioCumulativeBreakTime = 0;
                audioCumulativeDecodeTime = 0;
                audioCumulativeBreakRate = 0;
                videoCumulativeBreakCount = 0;
                videoCumulativeBreakTime = 0;
                videoCumulativeDecodeTime = 0;
                videoCumulativeBreakRate = 0;
            }
        };

		enum ZegoMixSysPlayoutPropertyMask
		{
			MIX_PROP_NONE = 0,
			MIX_PROP_ENABLE_AGC_FOR_SYS_PLAYOUT = 1, //当开启采集系统声卡声音时，传递此参数启用自动增益(仅支持win) 
		};
    
        /**
         * 推流参数
         */
        struct ZegoPublishStreamParams
        {
            // 流 ID
            const char* streamID;
            // 直播名称
            const char* streamTitle;
            // 直播属性，参考 ZegoPublishFlag
            int publishFlag;
            // 推流参数
            const char* publishParams;
            // 推流 channel Index. 默认为主Channel
            AV::PublishChannelIndex channelIndex;
            // 关联的房间 ID，为 NULL 时，表示关联默认房间
            const char* roomID;
            //推流是否同步网络时间,仅RTC流有效，与SetStreamAlignmentProperty 接口配合使用, 用于混流时各路流的对齐, 常用于KTV场景。
            int forceSynchronousNetworkTime = 0;
            
            ZegoPublishStreamParams()
            {
                streamID = NULL;
                streamTitle = NULL;
                publishFlag = 0;
                publishParams = NULL;
                channelIndex = AV::PUBLISH_CHN_MAIN;
                roomID = NULL;
            }
        };
    
        /**
         * 拉流参数
         */
        struct ZegoPlayStreamParams
        {
            // 流 ID
            const char* streamID;
            // 用于拉流播放的 view
            void* displayView;
            // 可选，额外的信息
            ZegoStreamExtraPlayInfo* extraInfo;
            // 房间 ID
            const char* roomID;
            
            ZegoPlayStreamParams()
            {
                streamID = NULL;
                displayView = NULL;
                extraInfo = NULL;
                roomID = NULL;
            }
        };
        
        /**
         房间模式
         */
        enum ZegoRoomMode 
        {
            // 单房间模式
            ZegoRoomModeSingleRoom = 0, 
            // 多房间模式
            ZegoRoomModeMultiRoom = 1,
        };

        /**
        范围语音模式
        */
        enum ZegoRangeAudioMode
        {
            //面向所有人
            ZegoRangeAudioModeWorld = 1, 
            //仅小队
            ZegoRangeAudioModeTeam = 2 
        };

        /**
        范围语音麦克风状态
        */
        enum ZegoRangeAudioMicrophoneState
        {
            //正在打开中
            Opening = 1, 
            //打开成功
            Success   = 2,
            //临时中断，正在重试中
            TempBroken    = 3, 
            // 打开失败
            Failed      = 4, 
        };
    }
}

#endif /* ZegoLiveRoomDefines_h */
