//
//  zego-api-audio-vad.h
//  ZegoLiveRoom
//
//  Created by kebo on 2021/1/14.
//

#ifndef zego_api_audio_vad_hpp
#define zego_api_audio_vad_hpp

#include <stddef.h>
#include "zego-api-defines.h"

namespace ZEGO
{
namespace AUDIOVAD
{

/**
 * 检测音频包类
 * 
 */
struct ZegoAudioVADClient;

/**
检测结果类型
*/
enum ZegoAudioVADType
{
    // 噪声
    ZegoAudioVADTypeNoise = 0,
    // 语音
    ZegoAudioVADTypeSpeech = 1,
};

/**
 * 创建检测音频包类对象
 * 
 * @return 检测音频包类对象指针
 * @note 每条流应该对应一个检测音频包类对象，或者将已有对象重置后再应用于不同的流
 */
ZEGOAVKIT_API ZegoAudioVADClient* CreateZegoAudioVADClient();

/**
 * 销毁检测音频包类对象
 * 
 * @param client 检测音频包类对象指针
 */
ZEGOAVKIT_API void DestroyZegoAudioVADClient(ZegoAudioVADClient* client);

/**
 * 检测音频包是否包含语音
 * 
 * @param client 检测音频包类对象指针
 * @param buf 音频数据
 * @param bufLen 音频数据长度
 * @param sampleRate 采样率
 * @param channels 通道数
 * @return 返回值参考 ZegoAudioVADType 定义
 */
ZEGOAVKIT_API ZegoAudioVADType ZegoAudioVADClientUpdate(ZegoAudioVADClient* client, unsigned char* buf, int bufLen, int sampleRate, int channels);

/**
 * 重置检测音频包类对象
 * 
 * @param client 检测音频包类对象
 * @note 当音频数据不连续时，需要调用该接口重置状态。
 * @note 在切换流之后应该重置
 * @note 当发生音频数据丢失或丢弃时应该重置
 * @return 0 表示成功，-1 表示失败
 */
ZEGOAVKIT_API int ZegoAudioVADClientReset(ZegoAudioVADClient* client);

}// AUDIOVAD
}// ZEGO

#endif /* zego_api_audio_vad_hpp */
