#ifndef zego_api_peertopeer_latency_probe_h
#define zego_api_peertopeer_latency_probe_h
#include "zego-api-defines.h"

namespace ZEGO
{
    namespace PEERTOPEERLATENCYPROBE
    {
        class IZegoPeerToPeerLatencyProbeCallback
        {
        public:
            /**
            延迟测量回调 收到推流端延迟数据时回调

            @param pszStreamID 流 ID
            @param delay 大于0为延迟(ms) -1为推流端时间同步异常 -2为拉流端时间同步异常
            */
            virtual void OnPeerToPeerLatencyProbeResult(const char* pszStreamID, int delay) = 0;

            virtual ~IZegoPeerToPeerLatencyProbeCallback() {}

        };
        /**
         设置拉流端延迟测量模块的回调
         @param  callback 指针
         */
        ZEGOAVKIT_API void SetPeerToPeerLatencyProbeCallback(IZegoPeerToPeerLatencyProbeCallback* pCallback);

        /**
         推流端激活延迟测量功能，默认时间间隔为3秒
         
         @attention 必须在调用初始化 SDK 接口之后、推流接口之前设置。

         @param  enable true开启 false关闭
         @param  idx   推流通道 index，请参考 AV::PublishChannelIndex
         */
        ZEGOAVKIT_API void EnablePeerToPeerLatencyProbe(bool enable, AV::PublishChannelIndex chn = AV::PUBLISH_CHN_MAIN);

        /**
         推流端设置延迟数据发送时间间隔毫秒数，默认为3000

         @attention 必须在调用初始化 SDK 接口之后、推流接口之前设置。

         @param  interval 时间间隔，取值范围[1000,300000]
         @param  idx   推流通道 index，请参考 AV::PublishChannelIndex
         */
        ZEGOAVKIT_API void SetPeerToPeerLatencyProbeInterval(int interval, AV::PublishChannelIndex chn = AV::PUBLISH_CHN_MAIN);
    }
}

#endif /* zego_api_peertopeer_latency_probe_h */
